/*
    Copyright (C) 2004, 2005, 2007 Nikolas Zimmermann <zimmermann@kde.org>
                  2004, 2005, 2006 Rob Buis <buis@kde.org>

    This file is part of the KDE project

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "config.h"

#if ENABLE(SVG) && ENABLE(SVG_FILTERS)
#include "SVGFETurbulenceElement.h"

#include "SVGParserUtilities.h"
#include "SVGResourceFilter.h"

namespace WebCore {

SVGFETurbulenceElement::SVGFETurbulenceElement(const QualifiedName& tagName, Document* doc)
    : SVGFilterPrimitiveStandardAttributes(tagName, doc)
    , m_baseFrequencyX(0.0f)
    , m_baseFrequencyY(0.0f)
    , m_numOctaves(1)
    , m_seed(0.0f)
    , m_stitchTiles(SVG_STITCHTYPE_NOSTITCH)
    , m_type(SVG_TURBULENCE_TYPE_TURBULENCE)
    , m_filterEffect(0)
{
}

SVGFETurbulenceElement::~SVGFETurbulenceElement()
{
}

ANIMATED_PROPERTY_DEFINITIONS_WITH_CUSTOM_IDENTIFIER(SVGFETurbulenceElement, float, BaseFrequencyX, baseFrequencyX, SVGNames::baseFrequencyAttr, "baseFrequencyX")
ANIMATED_PROPERTY_DEFINITIONS_WITH_CUSTOM_IDENTIFIER(SVGFETurbulenceElement, float, BaseFrequencyY, baseFrequencyY, SVGNames::baseFrequencyAttr, "baseFrequencyY")
ANIMATED_PROPERTY_DEFINITIONS(SVGFETurbulenceElement, float, Seed, seed, SVGNames::seedAttr)
ANIMATED_PROPERTY_DEFINITIONS(SVGFETurbulenceElement, long, NumOctaves, numOctaves, SVGNames::numOctavesAttr)
ANIMATED_PROPERTY_DEFINITIONS(SVGFETurbulenceElement, int, StitchTiles, stitchTiles, SVGNames::stitchTilesAttr)
ANIMATED_PROPERTY_DEFINITIONS(SVGFETurbulenceElement, int, Type, type, SVGNames::typeAttr)

void SVGFETurbulenceElement::parseMappedAttribute(MappedAttribute* attr)
{
    const String& value = attr->value();
    if (attr->name() == SVGNames::typeAttr) {
        if (value == "fractalNoise")
            setTypeBaseValue(SVG_TURBULENCE_TYPE_FRACTALNOISE);
        else if (value == "turbulence")
            setTypeBaseValue(SVG_TURBULENCE_TYPE_TURBULENCE);
    } else if (attr->name() == SVGNames::stitchTilesAttr) {
        if (value == "stitch")
            setStitchTilesBaseValue(SVG_STITCHTYPE_STITCH);
        else if (value == "nostitch")
            setStitchTilesBaseValue(SVG_STITCHTYPE_NOSTITCH);
    } else if (attr->name() == SVGNames::baseFrequencyAttr) {
        float x, y;
        if (parseNumberOptionalNumber(value, x, y)) {
            setBaseFrequencyXBaseValue(x);
            setBaseFrequencyYBaseValue(y);
        }
    } else if (attr->name() == SVGNames::seedAttr)
        setSeedBaseValue(value.toFloat());
    else if (attr->name() == SVGNames::numOctavesAttr)
        setNumOctavesBaseValue(value.toUIntStrict());
    else
        SVGFilterPrimitiveStandardAttributes::parseMappedAttribute(attr);
}

SVGFETurbulence* SVGFETurbulenceElement::filterEffect(SVGResourceFilter* filter) const
{
    
    if (!m_filterEffect)
        m_filterEffect = SVGFETurbulence::create(filter);
    
    m_filterEffect->setType((SVGTurbulanceType) type());
    m_filterEffect->setBaseFrequencyX(baseFrequencyX());
    m_filterEffect->setBaseFrequencyY(baseFrequencyY());
    m_filterEffect->setNumOctaves(numOctaves());
    m_filterEffect->setSeed(seed());
    m_filterEffect->setStitchTiles(stitchTiles() == SVG_STITCHTYPE_STITCH);

    setStandardAttributes(m_filterEffect.get()); 
    return m_filterEffect.get();
}

}

#endif // ENABLE(SVG)

// vim:ts=4:noet
